@@ -51,15 +51,12 @@
 // view in the message header pane.
 ////////////////////////////////////////////////////////////////////////////////////
 
-if(!org) var org={};
-if(!org.mozdev) org.mozdev={};
-if(!org.mozdev.compactHeader) org.mozdev.compactHeader = {};
 
-//Components.utils.import("chrome://CompactHeader/content/RSSLinkify.jsm");
-//Components.utils.import("chrome://CompactHeader/content/debug.jsm");
-//Components.utils.import("chrome://CompactHeader/content/toolbar.jsm");
+if (typeof org_mozdev_compactHeader == "undefined") {
+  var org_mozdev_compactHeader = {};
+};
 
-org.mozdev.compactHeader.pane = function() {
+org_mozdev_compactHeader.pane = function() {
   var pub = {};
 
   const COMPACTHEADER_EXTENSION_UUID = "{58D4392A-842E-11DE-B51A-C7B855D89593}";
@@ -90,6 +87,9 @@ org.mozdev.compactHeader.pane = function() {
     {name:"date", outputFunction:coheUpdateDateValue}
     ];
 
+  var prefserv = Components.classes["@mozilla.org/preferences-service;1"]
+    .getService(Components.interfaces.nsIPrefBranch);
+
   var cohePrefBranch = Components.classes["@mozilla.org/preferences-service;1"]
     .getService(Components.interfaces.nsIPrefService)
     .getBranch("extensions.CompactHeader.");
@@ -107,6 +107,9 @@ org.mozdev.compactHeader.pane = function() {
   var coheFirstTime = true;
   var headerFirstTime = true;
 
+  var pressMores = null;
+  var gMoreTooltip = "";
+
   function coheOutputSubject(headerEntry, headerValue) {
     var subjectBox;
 
@@ -132,6 +135,11 @@ org.mozdev.compactHeader.pane = function() {
     var names = {};
     var numAddresses =  0;
 
+    let moreButton = document.getAnonymousElementByAttribute(document.getElementById("CompactHeader_collapsed2LtoCcBccBox"),
+        "anonid", "more");
+    let moreTooltip = gMoreTooltip;
+//    let moreTooltip = moreButton.getAttribute("tooltiptext");
+
     var msgHeaderParser = Components.classes["@mozilla.org/messenger/headerparser;1"]
                                     .getService(Components.interfaces.nsIMsgHeaderParser);
     numAddresses = msgHeaderParser.parseHeadersWithArray(emailAddresses, addresses, names, fullNames);
@@ -151,6 +159,19 @@ org.mozdev.compactHeader.pane = function() {
       } else {
         address.displayName = names.value[index];
       }
+      org_mozdev_compactHeader.debug.log("0: " + address.fullAddress);
+      org_mozdev_compactHeader.debug.log("0: " + addressType);
+      if (address.fullAddress != "" &&
+           (addressType == "to" || addressType == "cc" || addressType == "bcc")) {
+        if (moreTooltip == "") {
+          moreTooltip = address.fullAddress;
+          org_mozdev_compactHeader.debug.log("1: " + address.fullAddress);
+        } else {
+          moreTooltip = moreTooltip + ", " + address.fullAddress;
+          org_mozdev_compactHeader.debug.log("2: " + address.fullAddress);
+        }
+      }
+//      window.alert(address);
       if (headerEntry.useToggle && (typeof headerEntry.enclosingBox.addAddressView == 'function')) {
         headerEntry.enclosingBox.addAddressView(address);
       } else {
@@ -158,6 +179,9 @@ org.mozdev.compactHeader.pane = function() {
       }
       index++;
     }
+    org_mozdev_compactHeader.debug.log("tooltiptext: " + moreTooltip);
+    moreButton.setAttribute("tooltiptext", moreTooltip);
+    gMoreTooltip = moreTooltip;
 
     if (headerEntry.useToggle && (typeof headerEntry.enclosingBox.buildViews == 'function'))
       headerEntry.enclosingBox.buildViews();
@@ -172,14 +196,25 @@ org.mozdev.compactHeader.pane = function() {
 
   function coheInitializeHeaderViewTables()
   {
+    org_mozdev_compactHeader.debug.log("coheInitializeHeaderViewTables start");
     gCoheCollapsedHeaderView = {};
     var index;
 
-    if (cohePrefBranch.getBoolPref("headersize.twolineview")) {
+    if (prefserv.getBoolPref("extensions.CompactHeader.headersize.twolineview")) {
       for (index = 0; index < gCoheCollapsedHeader2LListLongAddresses.length; index++) {
         gCoheCollapsedHeaderView[gCoheCollapsedHeader2LListLongAddresses[index].name] =
           new createHeaderEntry('CompactHeader_collapsed2L', gCoheCollapsedHeader2LListLongAddresses[index]);
       }
+      let moreButton = document.getAnonymousElementByAttribute(document.getElementById("CompactHeader_collapsed2LtoCcBccBox"),
+        "anonid", "more");
+
+      if (moreButton) {
+        let oldToggleWrap = moreButton.parentNode.toggleWrap;
+        moreButton.parentNode.toggleWrap = function() {
+          pressMores = pressMoreButtons;
+          pub.coheToggleHeaderView();
+        };
+      }
     } else {
       for (index = 0; index < gCoheCollapsedHeader1LListLongAddresses.length; index++) {
         gCoheCollapsedHeaderView[gCoheCollapsedHeader1LListLongAddresses[index].name] =
@@ -187,11 +222,36 @@ org.mozdev.compactHeader.pane = function() {
       }
     }
 
-    org.mozdev.compactHeader.RSSLinkify.InitializeHeaderViewTables();
+    org_mozdev_compactHeader.debug.log("call to org_mozdev_compactHeader.RSSLinkify.InitializeHeaderViewTables");
+    org_mozdev_compactHeader.RSSLinkify.InitializeHeaderViewTables();
+
+    org_mozdev_compactHeader.debug.log("coheInitializeHeaderViewTables stop");
+  }
+
+  function pressMoreButtons() {
+    let moreButtonTo = document.getAnonymousElementByAttribute(document.getElementById("expandedtoBox"),
+        "anonid", "more");
+    let moreButtonCC = document.getAnonymousElementByAttribute(document.getElementById("expandedccBox"),
+        "anonid", "more");
+    let moreButtonBCC = document.getAnonymousElementByAttribute(document.getElementById("expandedbccBox"),
+        "anonid", "more");
+    if (!moreButtonTo.hasAttribute("collapsed")) {
+      moreButtonTo.click();
+      org_mozdev_compactHeader.debug.log("toggle To");
+    }
+    if (!moreButtonCC.hasAttribute("collapsed")) {
+      moreButtonCC.click();
+      org_mozdev_compactHeader.debug.log("toggle cc");
+    }
+    if (!moreButtonBCC.hasAttribute("collapsed")) {
+      moreButtonBCC.click();
+      org_mozdev_compactHeader.debug.log("toggle bcc");
+    }
+    pressMores = null;
   }
 
   pub.coheOnLoadMsgHeaderPane = function() {
-    org.mozdev.compactHeader.debug.log("coheOnLoadMsgHeaderPane start");
+    org_mozdev_compactHeader.debug.log("coheOnLoadMsgHeaderPane start");
 
     coheInitializeHeaderViewTables();
 
@@ -207,7 +267,7 @@ org.mozdev.compactHeader.pane = function() {
     gCoheCollapsedHeaderViewMode =
       deckHeaderView.selectedPanel == document.getElementById('CompactHeader_collapsedHeaderView');
 
-    org.mozdev.compactHeader.debug.log("coheOnLoadMsgHeaderPane 1");
+    org_mozdev_compactHeader.debug.log("coheOnLoadMsgHeaderPane 1");
 
     // work around XUL deck bug where collapsed header view, if it's the persisted
     // default, wouldn't be sized properly because of the larger expanded
@@ -225,44 +285,60 @@ org.mozdev.compactHeader.pane = function() {
       document.getElementById('CompactHeader_collapsed2LHeadersBox').collapsed = true;
     }
 
-    org.mozdev.compactHeader.debug.log("coheOnLoadMsgHeaderPane 2");
+    org_mozdev_compactHeader.debug.log("coheOnLoadMsgHeaderPane 2");
 
     if (coheFirstTime)
     {
-      org.mozdev.compactHeader.debug.log("coheFirstTime");
+      org_mozdev_compactHeader.debug.log("coheFirstTime");
       coheFirstTime = false;
       gMessageListeners.push(coheMessageListener);
-      org.mozdev.customizeHeaderToolbar.messenger.loadToolboxData();
-      org.mozdev.compactHeader.toolbar.fillToolboxPalette();
-      org.mozdev.customizeHeaderToolbar.messenger.saveToolboxData();
+      org_mozdev_compactHeader.messenger.loadToolboxData();
+      org_mozdev_compactHeader.toolbar.fillToolboxPalette();
+      org_mozdev_compactHeader.messenger.saveToolboxData();
 
       let collapsed2LtoCcBccBox = document.getElementById("CompactHeader_collapsed2LtoCcBccBox");
       if (collapsed2LtoCcBccBox) {
         let updateEmailAddressNodeFunction = collapsed2LtoCcBccBox.updateEmailAddressNode;
-        function updateEmailAddressNodeNew(aEmailNode, aAddress) {
-          updateEmailAddressNode(aEmailNode, aAddress)
+        collapsed2LtoCcBccBox.updateEmailAddressNode = function(aEmailNode, aAddress) {
+          try {
+            updateEmailAddressNodeFunction(aEmailNode, aAddress);
+          }
+          catch(e) {
+            org_mozdev_compactHeader.debug.log("got execption " + e +
+              " from updateEmailAddressNode");
+          }
           aEmailNode.setAttribute("addressType", aAddress.addressType);
+        };
+        if (typeof collapsed2LtoCcBccBox.setNMoreTooltiptext == 'function') {
+          // remove setNMoreTooltiptext because we have our own function
+          collapsed2LtoCcBccBox.setNMoreTooltiptext = function() {
+            let moreButton = document.getAnonymousElementByAttribute(document.getElementById("CompactHeader_collapsed2LtoCcBccBox"),
+              "anonid", "more");
+            if (moreButton) {
+              moreButton.setAttribute("tooltiptext", gMoreTooltip);
+            }
+          };
         }
-        collapsed2LtoCcBccBox.updateEmailAddressNode = updateEmailAddressNodeNew;
       }
     }
 
-    org.mozdev.compactHeader.debug.log("coheOnLoadMsgHeaderPane 2a");
+    org_mozdev_compactHeader.debug.log("coheOnLoadMsgHeaderPane 2a");
 
     if (cohe.firstrun) {
       coheCheckFirstRun();
     }
 
-    org.mozdev.compactHeader.debug.log("coheOnLoadMsgHeaderPane 3");
+    org_mozdev_compactHeader.debug.log("coheOnLoadMsgHeaderPane 3");
+
+    org_mozdev_compactHeader.toolbar.setButtonStyle();
+    org_mozdev_compactHeader.messenger.saveToolboxData();
+    org_mozdev_compactHeader.toolbar.dispMUACheck();
+
+    org_mozdev_compactHeader.debug.log("coheOnLoadMsgHeaderPane 4");
 
     coheToggleHeaderContent();
 
-    org.mozdev.compactHeader.debug.log("coheOnLoadMsgHeaderPane 4");
-
-    org.mozdev.compactHeader.toolbar.setButtonStyle();
-    org.mozdev.customizeHeaderToolbar.messenger.saveToolboxData();
-    org.mozdev.compactHeader.toolbar.dispMUACheck();
-    org.mozdev.compactHeader.debug.log("coheOnLoadMsgHeaderPane stop");
+    org_mozdev_compactHeader.debug.log("coheOnLoadMsgHeaderPane stop");
   }
 
   var coheMessageListener =
@@ -276,11 +352,13 @@ org.mozdev.compactHeader.pane = function() {
     function cML_onEndHeaders() {
       ClearHeaderView(gCoheCollapsedHeaderView);
       coheUpdateMessageHeaders();
+      if (pressMores) {
+        pressMoreButtons();
+        pressMore = null;
+      }
     },
 
-    onEndAttachments: function cML_onEndAttachments() {
-      setTimeout(org.mozdev.compactHeader.toolbar.onChangeDispMUAicon, 50);
-    }
+    onEndAttachments: function cML_onEndAttachments(){}
   };
 
   pub.coheOnUnloadMsgHeaderPane = function()
@@ -289,8 +367,10 @@ org.mozdev.compactHeader.pane = function() {
               .getService(Components.interfaces.nsIAbManager)
               .removeAddressBookListener(coheAddressBookListener);
 
-    removeEventListener('messagepane-loaded', coheOnLoadMsgHeaderPane, true);
-    removeEventListener('messagepane-unloaded', coheOnUnloadMsgHeaderPane, true);
+    removeEventListener('messagepane-loaded',
+    pub.coheOnLoadMsgHeaderPane, true);
+    removeEventListener('messagepane-unloaded',
+      pub.coheOnUnloadMsgHeaderPane, true);
   }
 
   var coheAddressBookListener =
@@ -335,17 +415,17 @@ org.mozdev.compactHeader.pane = function() {
     if (gCoheCollapsedHeaderViewMode)
       showHeaderView(gCoheCollapsedHeaderView);
 
-    org.mozdev.compactHeader.RSSLinkify.UpdateHeaderView(currentHeaderData);
+    org_mozdev_compactHeader.RSSLinkify.UpdateHeaderView(currentHeaderData);
 
     if (cohePrefBranch.getBoolPref("headersize.addressstyle")) {
       selectEmailDisplayed();
     }
 
-    //org.mozdev.compactHeader.toolbar.fillToolboxPalette(document);
+    //org_mozdev_compactHeader.toolbar.fillToolboxPalette(document);
     coheToggleHeaderContent();
-    org.mozdev.compactHeader.toolbar.CHTUpdateReplyButton();
-    org.mozdev.compactHeader.toolbar.CHTUpdateJunkButton();
-    org.mozdev.compactHeader.buttons.coheToggleStar();
+    org_mozdev_compactHeader.toolbar.CHTUpdateReplyButton();
+    org_mozdev_compactHeader.toolbar.CHTUpdateJunkButton();
+    org_mozdev_compactHeader.buttons.coheToggleStar();
   }
 
   function enableButtons() {
@@ -359,6 +439,7 @@ org.mozdev.compactHeader.pane = function() {
   }
 
   pub.coheToggleHeaderView = function() {
+    org_mozdev_compactHeader.debug.log("coheToggleHeaderView start");
     gCoheCollapsedHeaderViewMode = !gCoheCollapsedHeaderViewMode;
 
     let deck = document.getElementById('msgHeaderViewDeck');
@@ -368,12 +449,13 @@ org.mozdev.compactHeader.pane = function() {
 
     if (gCoheCollapsedHeaderViewMode) {
       deck.selectedPanel = document.getElementById("CompactHeader_collapsedHeaderView")
-      coheUpdateMessageHeaders();
+      gDBView.reloadMessage();
+      //coheUpdateMessageHeaders();
     } else {
       deck.selectedPanel = document.getElementById("expandedHeaderView");
-      ClearHeaderView(gExpandedHeaderView);
-      UpdateExpandedMessageHeaders();
-      //gDBView.reloadMessage();
+      //ClearHeaderView(gExpandedHeaderView);
+      gDBView.reloadMessage();
+      //UpdateExpandedMessageHeaders();
     }
 
     // Work around a xul deck bug where the height of the deck is determined
@@ -382,9 +464,11 @@ org.mozdev.compactHeader.pane = function() {
     syncGridColumnWidths();
 
     coheToggleHeaderContent();
+    org_mozdev_compactHeader.debug.log("coheToggleHeaderView stop");
   }
 
   function coheToggleHeaderContent() {
+    org_mozdev_compactHeader.debug.log("coheToggleHeaderContent start");
     var strHideLabel = document.getElementById("CompactHeader_CoheHideDetailsLabel").value;
     var strShowLabel = document.getElementById("CompactHeader_CoheShowDetailsLabel").value;
     var strLabel;
@@ -428,23 +512,27 @@ org.mozdev.compactHeader.pane = function() {
       }
     }
 
-    org.mozdev.customizeHeaderToolbar.messenger.loadToolboxData();
+    org_mozdev_compactHeader.messenger.loadToolboxData();
 
     if (gCoheCollapsedHeaderViewMode) {
       strLabel = strShowLabel;
     } else {
       strLabel = strHideLabel;
     }
+    if (document.getElementById("CompactHeader_hideDetailsMenu")) {
+      document.getElementById("CompactHeader_hideDetailsMenu").setAttribute("label", strLabel);
+    }
+
+    org_mozdev_compactHeader.toolbar.setCurrentToolboxPosition(gCoheCollapsedHeaderViewMode);
 
     if (document.getElementById("CompactHeader_hideDetailsMenu")) {
       document.getElementById("CompactHeader_hideDetailsMenu").setAttribute("label", strLabel);
     }
 
-    org.mozdev.compactHeader.toolbar.toggle(gCoheCollapsedHeaderViewMode);
+    document.getElementById("CompactHeader_viewMenuCompactBroadcast")
+            .setAttribute("checked", gCoheCollapsedHeaderViewMode);
 
-    if (document.getElementById("CompactHeader_hideDetailsMenu")) {
-      document.getElementById("CompactHeader_hideDetailsMenu").setAttribute("label", strLabel);
-    }
+    org_mozdev_compactHeader.debug.log("coheToggleHeaderContent stop");
   }
 
   // default method for updating a header value into a header entry
@@ -453,13 +541,18 @@ org.mozdev.compactHeader.pane = function() {
     headerEntry.textNode.value = headerValue;
   }
 
-  function coheUpdateDateValue(headerEntry, headerValue) {
+  function coheUpdateDateValue(headerEntry, headerValue, dummy, currentHeaderData) {
     //var t = currentHeaderData.date.headerValue;
-    var d
-    d = document.getElementById("CompactHeader_collapsed1LdateBox");
-    d.textContent = headerValue;
-    d = document.getElementById("CompactHeader_collapsed2LdateBox");
-    d.textContent = headerValue;
+    var d1, d2;
+    d1 = document.getElementById("CompactHeader_collapsed1LdateBox");
+    d2 = document.getElementById("CompactHeader_collapsed2LdateBox");
+    if ("x-mozilla-localizeddate" in currentHeaderData) {
+      d1.textContent = currentHeaderData["x-mozilla-localizeddate"].headerValue;
+      d2.textContent = currentHeaderData["x-mozilla-localizeddate"].headerValue;
+    } else {
+      d1.textContent = headerValue;
+      d2.textContent = headerValue;
+    }
   }
 
 
@@ -468,15 +561,31 @@ org.mozdev.compactHeader.pane = function() {
   // view care about this header value. if it does then call updateHeaderEntry
   function coheUpdateMessageHeaders()
   {
+    org_mozdev_compactHeader.debug.log("coheUpdateMessageHeaders start");
     // Remove the height attr so that it redraws correctly. Works around a
     // problem that attachment-splitter causes if it's moved high enough to
     // affect the header box:
     document.getElementById('msgHeaderView').removeAttribute('height');
 
+    let moreButton = document.getAnonymousElementByAttribute(document.getElementById("CompactHeader_collapsed2LtoCcBccBox"),
+        "anonid", "more");
+    moreButton.setAttribute("tooltiptext", "");
+    gMoreTooltip = "";
+
     // iterate over each header we received and see if we have a matching entry
     // in each header view table...
-    for (var headerName in currentHeaderData)
+    var keys = [];
+    for (var key in currentHeaderData) {
+      if (currentHeaderData.hasOwnProperty(key)) {
+        keys.push(key);
+      }
+    }
+
+    keys.sort();
+    keys.reverse();
+    for (let i=0; i<keys.length; i++)
     {
+      let headerName = keys[i];
       var headerField = currentHeaderData[headerName];
       var headerEntry = null;
 
@@ -489,31 +598,31 @@ org.mozdev.compactHeader.pane = function() {
       }
 
       if (headerEntry) {
-        headerEntry.outputFunction(headerEntry, headerField.headerValue, headerName);
+        headerEntry.outputFunction(headerEntry, headerField.headerValue, headerName, currentHeaderData);
         headerEntry.valid = true;
       }
     }
 
     if (headerFirstTime) {
-      org.mozdev.compactHeader.debug.log("headerFirstTime");
+      org_mozdev_compactHeader.debug.log("headerFirstTime");
       headerFirstTime = false;
       var toolbox = document.getElementById("header-view-toolbox");
       var mailToolbox = document.getElementById("mail-toolbox");
       var oldCustomizeDone = toolbox.customizeDone;
       var oldCustomizeDoneMailToolbox = mailToolbox.customizeDone;
       toolbox.customizeDone = function(aEvent) {
-        org.mozdev.compactHeader.debug.log("customizeDone start");
+        org_mozdev_compactHeader.debug.log("customizeDone start");
         oldCustomizeDone(aEvent);
-        org.mozdev.compactHeader.debug.log("customizeDone 0");
-        org.mozdev.compactHeader.toolbar.onDoCustomizationHeaderViewToolbox("doCustomization");
-        org.mozdev.compactHeader.debug.log("customizeDone stop");
+        org_mozdev_compactHeader.debug.log("customizeDone 0");
+        org_mozdev_compactHeader.toolbar.onDoCustomizationHeaderViewToolbox("doCustomization");
+        org_mozdev_compactHeader.debug.log("customizeDone stop");
       };
       mailToolbox.customizeDone = function(aEvent) {
-        org.mozdev.compactHeader.debug.log("customizeDone start");
+        org_mozdev_compactHeader.debug.log("customizeDone start");
         oldCustomizeDoneMailToolbox(aEvent);
-        org.mozdev.compactHeader.debug.log("customizeDone 0");
-        org.mozdev.compactHeader.toolbar.onDoCustomizationHeaderViewToolbox("doCustomization");
-        org.mozdev.compactHeader.debug.log("customizeDone stop");
+        org_mozdev_compactHeader.debug.log("customizeDone 0");
+        org_mozdev_compactHeader.toolbar.onDoCustomizationHeaderViewToolbox("doCustomization");
+        org_mozdev_compactHeader.debug.log("customizeDone stop");
       };
     }
 
@@ -522,6 +631,7 @@ org.mozdev.compactHeader.pane = function() {
 
     // now update the view to make sure the right elements are visible
     coheUpdateHeaderView();
+    org_mozdev_compactHeader.debug.log("coheUpdateMessageHeaders stop");
   }
 
   function selectEmailDisplayed() {
@@ -563,23 +673,26 @@ org.mozdev.compactHeader.pane = function() {
     }
   };
 
-  var myPrefObserverHeaderSize =
+  var myPrefObserver =
   {
     register: function()
     {
+      org_mozdev_compactHeader.debug.log("prefObserver registration start");
+
       // First we'll need the preference services to look for preferences.
       var prefService = Components.classes["@mozilla.org/preferences-service;1"]
                                   .getService(Components.interfaces.nsIPrefService);
 
       // For this._branch we ask that the preferences for extensions.myextension. and children
-      this._branch = prefService.getBranch("extensions.CompactHeader.headersize.");
+      this._branch = prefService.getBranch("extensions.CompactHeader.");
 
-      // Now we queue the interface called nsIPrefBranch2. This interface is described as:
+      // Now we queue the interface called nsIPrefBranch. This interface is described as:
       // "nsIPrefBranch2 allows clients to observe changes to pref values."
-      this._branch.QueryInterface(Components.interfaces.nsIPrefBranch2);
+      this._branch.QueryInterface(Components.interfaces.nsIPrefBranch);
 
       // Finally add the observer.
       this._branch.addObserver("", this, false);
+      org_mozdev_compactHeader.debug.log("prefObserver registration stop");
     },
 
     unregister: function()
@@ -590,58 +703,67 @@ org.mozdev.compactHeader.pane = function() {
 
     observe: function(aSubject, aTopic, aData)
     {
-      org.mozdev.compactHeader.debug.log("prefObserver start");
+      org_mozdev_compactHeader.debug.log("prefObserver start");
       if(aTopic != "nsPref:changed") return;
       // aSubject is the nsIPrefBranch we're observing (after appropriate QI)
       // aData is the name of the pref that's been changed (relative to aSubject)
-      org.mozdev.compactHeader.debug.log("prefObserver 1: " + aData);
+      org_mozdev_compactHeader.debug.log("prefObserver 1: " + aData);
 
-      if (  (aData == "addressstyle")
-          ||(aData == "twolineview")
-          ||(aData == "linkify")
+      if (  (aData == "headersize.addressstyle")
+          ||(aData == "headersize.twolineview")
+          ||(aData == "headersize.linkify")
+          ||(aData == "toolbox.position")
           ) {
         preferencesUpdate();
+      } else if (aData == "header.doubleclick") {
+        setDblClickHeaderEventHandler();
       }
 
-      org.mozdev.compactHeader.debug.log("prefObserver stop");
+      org_mozdev_compactHeader.debug.log("prefObserver stop");
     }
-  }
+  };
 
   var wasHere = false;
 
   function preferencesUpdate() {
-    org.mozdev.compactHeader.debug.log("preferencesUpdate " + wasHere);
+    org_mozdev_compactHeader.debug.log("preferencesUpdate " + wasHere);
     if (!browserPreferences.getBoolPref("instantApply")
         && wasHere)
       return;
-    org.mozdev.compactHeader.debug.log("preferencesUpdate 2");
+    org_mozdev_compactHeader.debug.log("preferencesUpdate 2");
     wasHere = true;
-    ReloadMessage();
-    pub.coheOnLoadMsgHeaderPane();
+    if ((typeof gDBView  != "undefined") && gDBView) {
+      gDBView.reloadMessage();
+      pub.coheOnLoadMsgHeaderPane();
+    }
+    org_mozdev_compactHeader.toolbar.setCurrentToolboxPosition(gCoheCollapsedHeaderViewMode);
 //    var event = document.createEvent('Events');
 //    event.initEvent('messagepane-loaded', false, true);
 //    var headerViewElement = document.getElementById("msgHeaderView");
 //    headerViewElement.dispatchEvent(event);
-    setTimeout(clearReloadTimeout, 250);
-    org.mozdev.compactHeader.debug.log("preferencesUpdate stop");
+    setTimeout(function() {
+        clearReloadTimeout();
+        return;
+      }, 250);
+    org_mozdev_compactHeader.debug.log("preferencesUpdate stop");
   }
 
   function clearReloadTimeout() {
     wasHere = false;
-    org.mozdev.compactHeader.debug.log("wasHere cleared");
+    org_mozdev_compactHeader.debug.log("wasHere cleared");
   }
 
   function coheCheckFirstRun() {
     var appInfo = Components.classes["@mozilla.org/xre/app-info;1"]
                                      .getService(Components.interfaces.nsIXULAppInfo);
-//    org.mozdev.compactHeader.debug.log("first run 0");
-    var debugLevel = org.mozdev.compactHeader.debug.getLogLevel();
-    org.mozdev.compactHeader.debug.log("firstrun 3");
-    org.mozdev.compactHeader.toolbar.populateEmptyToolbar();
+//    org_mozdev_compactHeader.debug.log("first run 0");
+    var debugLevel = org_mozdev_compactHeader.debug.getLogLevel();
+    org_mozdev_compactHeader.debug.log("firstrun 3");
+    org_mozdev_compactHeader.toolbar.populateEmptyToolbar();
     Components.utils.import("resource://gre/modules/AddonManager.jsm");
     AddonManager.getAddonByID(COMPACTHEADER_EXTENSION_UUID,
       function(myAddon) {
-        org.mozdev.compactHeader.debug.log("first run 2");
+        org_mozdev_compactHeader.debug.log("first run 2");
         cohe.version = "";
         cohe.firstrun = false;
         cohe.current = myAddon.version;
@@ -652,59 +774,157 @@ org.mozdev.compactHeader.pane = function() {
         } finally {
           //check for first run
           if (cohe.firstrun){
-            org.mozdev.compactHeader.debug.log("first run 2c");
-            org.mozdev.compactHeader.toolbar.CHTSetDefaultButtons();
+            org_mozdev_compactHeader.debug.log("first run 2c");
+            org_mozdev_compactHeader.toolbar.CHTSetDefaultButtons();
             cohePrefBranch.setBoolPref("firstrun",false);
             cohePrefBranch.setCharPref("version",cohe.current);
-            org.mozdev.compactHeader.debug.log("first run 2cc");
+            org_mozdev_compactHeader.debug.log("first run 2cc");
           }
           //check for upgrade
           if (cohe.version!=cohe.current && !cohe.firstrun){
             cohePrefBranch.setCharPref("version",cohe.current);
-            org.mozdev.compactHeader.debug.log("found version change");
+            org_mozdev_compactHeader.debug.log("found version change");
             // XXX
           }
           cohe.firstrun = false;
-          org.mozdev.compactHeader.debug.log("first run 2d");
+          org_mozdev_compactHeader.debug.log("first run 2d");
         }
       }
     );
-    org.mozdev.compactHeader.debug.log("firstrun 4");
+    org_mozdev_compactHeader.debug.log("firstrun 4");
   }
 
 
   pub.coheInitializeOverlay = function() {
-    // var gExtensionManager = Components.classes["@mozilla.org/extensions/manager;1"].getService(Components.interfaces.nsIExtensionManager);
-    // check if this is part of CompactHeader
-    // if ((gExtensionManager.getItemForID(COMPACTHEADER_EXTENSION_UUID) == null) || isAddonDisabled(COMPACTHEADER_EXTENSION_UUID)) {
-    //  return;
-    //}
-    org.mozdev.compactHeader.debug.log("before register");
+    removeEventListener('load', org_mozdev_compactHeader.pane.coheInitializeOverlay, false);
+    org_mozdev_compactHeader.debug.log("before register");
     coheUninstallObserver.register();
-    myPrefObserverHeaderSize.register();
-    org.mozdev.compactHeader.debug.log("register PrefObserver");
-    org.mozdev.compactHeader.debug.log("after register");
+    org_mozdev_compactHeader.debug.log("register PrefObserver");
+    myPrefObserver.register();
+    org_mozdev_compactHeader.debug.log("after register");
+    if ((typeof MessageDisplayWidget != "undefined") && MessageDisplayWidget) {
+      org_mozdev_compactHeader.debug.log("coheInitializeOverlay found MessageDisplayWidget");
+      var oldUpdateActiveMessagePane = MessageDisplayWidget.prototype._updateActiveMessagePane;
+      MessageDisplayWidget.prototype._updateActiveMessagePane = function() {
+        org_mozdev_compactHeader.debug.log("_updateActiveMessagePane start");
+        oldUpdateActiveMessagePane.call(this);
+        org_mozdev_compactHeader.toolbar.setCurrentToolboxPosition(gCoheCollapsedHeaderViewMode);
+        org_mozdev_compactHeader.debug.log("_updateActiveMessagePane stop");
+      };
+    }
+    else {
+      org_mozdev_compactHeader.debug.log("coheInitializeOverlay didn't find MessageDisplayWidget");
+    }
+
+    var multiMessage = document.getElementById("multimessage");
+    if (multiMessage) {
+      org_mozdev_compactHeader.debug.log("multiMessage " + multiMessage);
+      multiMessage.addEventListener("DOMContentLoaded", multiMessageLoaded, true);
+    }
+
+    addMessagePaneBoxFocusHandler();
+    setDblClickHeaderEventHandler();
+    org_mozdev_compactHeader.debug.log("coheInitializeOverlay stop");
+  };
+
+  function addMessagePaneBoxFocusHandler() {
+    let messagepanebox = document.getElementById("messagepanebox");
+    if (messagepanebox) {
+      messagepanebox.addEventListener("focus", messagePaneBoxFocus, true);
+      messagepanebox.addEventListener("blur", messagePaneBoxBlur, true);
+    }
+  }
+
+  var msgHeaderViewBackground;
+
+  function messagePaneBoxFocus(event) {
+    org_mozdev_compactHeader.debug.log("messagePaneBoxFocus start");
+    let msgHeaderView = document.getElementById("msgHeaderView");
+    let wintype = document.documentElement.getAttribute("windowtype");
+//    let tabmail = document.getElementById("tabmail");
+    if (cohePrefBranch.getBoolPref("header.darkenonfocus") &&
+        msgHeaderView && wintype && wintype == "mail:3pane" ) {
+//          && tabmail && tabmail.tabContainer.selectedIndex == 0) {
+      org_mozdev_compactHeader.debug.log("background: " +
+          msgHeaderViewBackground);
+      if (typeof msgHeaderViewBackground === "undefined") {
+        var style =
+          document.defaultView.getComputedStyle(msgHeaderView, null);
+        msgHeaderViewBackground = style.getPropertyValue("background-color");
+      }
+      org_mozdev_compactHeader.debug.log("style: " + style);
+      org_mozdev_compactHeader.debug.log("background: " +
+        msgHeaderViewBackground);
+      let newColor = darkenColor(msgHeaderViewBackground);
+      msgHeaderView.style.backgroundColor = newColor;
+      //       msgHeaderView.setAttribute('style', 'background-color:darkblue;');
+    }
+    org_mozdev_compactHeader.debug.log("messagePaneBoxFocus stop");
+  }
+
+  function darkenColor(color) {
+    var digits = /(.*?)rgb\((\d+), (\d+), (\d+)\)/.exec(color);
+
+    var red = parseInt(digits[2]);
+    var green = parseInt(digits[3]);
+    var blue = parseInt(digits[4]);
+
+    let factor = 0.9;
+
+    red = red * factor;
+    green = green * factor;
+    blue = blue * factor;
+
+    var rgb = blue | (green << 8) | (red << 16);
+    return digits[1] + '#' + rgb.toString(16);
+}
+
+  function messagePaneBoxBlur(event) {
+    let msgHeaderView = document.getElementById("msgHeaderView");
+    let wintype = document.documentElement.getAttribute("windowtype");
+    let tabmail = document.getElementById("tabmail");
+//    if (msgHeaderView && wintype && wintype == "mail:3pane" &&
+//        tabmail && tabmail.tabContainer.selectedIndex == 0) {
+    if (msgHeaderView) {
+      if (!(typeof msgHeaderViewBackground === "undefined")) {
+        msgHeaderView.style.backgroundColor = msgHeaderViewBackground;
+        msgHeaderViewBackground = undefined;
+      }
+    }
+  }
+
+  function setDblClickHeaderEventHandler() {
+    var msgHeaderViewDeck = document.getElementById("msgHeaderViewDeck");
+    if (msgHeaderViewDeck){
+      org_mozdev_compactHeader.debug.log("msgHeaderViewDeck " + msgHeaderViewDeck);
+      if (cohePrefBranch.getBoolPref("header.doubleclick"))
+        msgHeaderViewDeck.addEventListener("dblclick", pub.coheToggleHeaderView, true);
+      else
+        msgHeaderViewDeck.removeEventListener("dblclick", pub.coheToggleHeaderView, true);
+    }
+  }
+
+  function multiMessageLoaded() {
+    var multiMessage = document.getElementById("multimessage");
+    if (multiMessage) {
+      multiMessage.removeEventListener("DOMContentLoaded", multiMessageLoaded, false);
+    }
+
+    org_mozdev_compactHeader.debug.log("multiMessageLoaded start");
+    org_mozdev_compactHeader.toolbar.setCurrentToolboxPosition(gCoheCollapsedHeaderViewMode);
+    org_mozdev_compactHeader.debug.log("multiMessageLoaded stop");
   }
 
   var coheUninstallObserver = {
     _uninstall : false,
     observe : function(subject, topic, data) {
-      if (topic == "em-action-requested") {
-        subject.QueryInterface(Components.interfaces.nsIUpdateItem);
-
-        if (subject.id == COMPACTHEADER_EXTENSION_UUID) {
-          org.mozdev.compactHeader.debug.log("uninstalling COHE 1");
-          if (data == "item-uninstalled") {
-            this._uninstall = true;
-          } else if (data == "item-cancel-action") {
-            this._uninstall = false;
-          }
-        }
-      } else if (topic == "quit-application-granted") {
-        org.mozdev.compactHeader.debug.log("uninstalling COHE 2");
+      org_mozdev_compactHeader.debug.log("test: " + subject + ", " +
+          topic + ", " + data);
+      if (topic == "quit-application-granted") {
+        org_mozdev_compactHeader.debug.log("uninstalling COHE 2");
         if (this._uninstall) {
           cohePrefBranch.deleteBranch("");
-          org.mozdev.compactHeader.toolbar.CHTCleanupButtons();
+          org_mozdev_compactHeader.toolbar.CHTCleanupButtons();
         }
         this.unregister();
       }
@@ -722,19 +942,19 @@ org.mozdev.compactHeader.pane = function() {
     },
 
     register : function() {
-      org.mozdev.compactHeader.debug.log("register uninstall start");
+      org_mozdev_compactHeader.debug.log("register uninstall start");
       var observerService =
         Components.classes["@mozilla.org/observer-service;1"].
         getService(Components.interfaces.nsIObserverService);
-      org.mozdev.compactHeader.debug.log("register uninstall start 1");
+      org_mozdev_compactHeader.debug.log("register uninstall start 1");
 
-      org.mozdev.compactHeader.debug.log("register uninstall start 2");
+      org_mozdev_compactHeader.debug.log("register uninstall start 2");
 
-      org.mozdev.compactHeader.debug.log("register uninstall neu 2");
+      org_mozdev_compactHeader.debug.log("register uninstall neu 2");
       observerService.addObserver(this, "quit-application-granted", false);
       Components.utils.import("resource://gre/modules/AddonManager.jsm");
       AddonManager.addAddonListener(this);
-      org.mozdev.compactHeader.debug.log("register uninstall neu 2");
+      org_mozdev_compactHeader.debug.log("register uninstall neu 2");
     },
     unregister : function() {
       var observerService =
@@ -746,28 +966,9 @@ org.mozdev.compactHeader.pane = function() {
     }
   }
 
-  function isAddonDisabled(uuid) {
-    var rdfService = Components.classes["@mozilla.org/rdf/rdf-service;1"].getService(Components.interfaces.nsIRDFService);
-    var addon = rdfService.GetResource("urn:mozilla:item:" + uuid);
-
-    var em = Components.classes["@mozilla.org/extensions/manager;1"].getService(Components.interfaces.nsIExtensionManager);
-    var ds = em.datasource;
-
-    var appRes = rdfService.GetResource("http://www.mozilla.org/2004/em-rdf#appDisabled");
-    var appDisabled = ds.GetTarget(addon, appRes, true);
-    if(appDisabled instanceof Components.interfaces.nsIRDFLiteral && appDisabled.Value == "true")
-      return true;
-
-    var userRes = rdfService.GetResource("http://www.mozilla.org/2004/em-rdf#userDisabled");
-    var userDisabled = ds.GetTarget(addon, userRes, true);
-    if(userDisabled instanceof Components.interfaces.nsIRDFLiteral && userDisabled.Value == "true")
-      return true;
-
-    return false;
-  }
   return pub;
 }();
 
-addEventListener('messagepane-loaded', org.mozdev.compactHeader.pane.coheOnLoadMsgHeaderPane, true);
-addEventListener('messagepane-unloaded', org.mozdev.compactHeader.pane.coheOnUnloadMsgHeaderPane, true);
-addEventListener('load', org.mozdev.compactHeader.pane.coheInitializeOverlay, false);
+addEventListener('messagepane-loaded', org_mozdev_compactHeader.pane.coheOnLoadMsgHeaderPane, true);
+addEventListener('messagepane-unloaded', org_mozdev_compactHeader.pane.coheOnUnloadMsgHeaderPane, true);
+addEventListener('load', org_mozdev_compactHeader.pane.coheInitializeOverlay, false);
