@@ -38,24 +38,68 @@
 # ***** END LICENSE BLOCK *****
 */
 
-EXPORTED_SYMBOLS = ["org"];
-
 //Components.utils.import("chrome://CompactHeader/content/debug.jsm");
 
-if(!org) var org={};
-if(!org.mozdev) org.mozdev={};
-if(!org.mozdev.compactHeader) org.mozdev.compactHeader = {};
+if (typeof org_mozdev_compactHeader == "undefined") {
+  var org_mozdev_compactHeader = {};
+};
 
 
-org.mozdev.compactHeader.toolbar = function() {
+org_mozdev_compactHeader.toolbar = function() {
   var pub = {};
 
   var cohePrefBranch = Components.classes["@mozilla.org/preferences-service;1"]
                                           .getService(Components.interfaces.nsIPrefService)
                                           .getBranch("extensions.CompactHeader.");
 
+  var gToolboxes = [
+    {pos:"top",   id:"CompactHeader_collapsed2LButtonBox", orient:"horizontal", header:"compact"},
+    {pos:"top",   id:"expandedBoxSpacer",                  orient:"horizontal", header:"expanded"},
+    {pos:"left",  id:"CompactHeader_leftSidebar_dummy",    orient:"vertical"},
+    {pos:"right", id:"CompactHeader_rightSidebar_dummy",   orient:"vertical"},
+    {pos:"none",  id:"",                                   orient:""},
+  ];
+
+  let gOtherMenuCommands = {
+      otherActionsOpenConversation: "cmd_openConversation",
+      otherActionsOpenInNewWindow:  "cmd_openMessage",
+      otherActionsOpenInNewTab:     "cmd_openMessage",
+      viewSourceMenuItem:           "cmd_viewPageSource",
+      markAsReadMenuItem:           "cmd_markAsRead",
+      markAsUnreadMenuItem:         "cmd_markAsUnread",
+      saveAsMenuItem:               "cmd_saveAsFile",
+      otherActionsPrint:            "cmd_print"
+    };
+
+  pub.cannotMoveToolbox = function() {
+    var appInfo = Components.classes["@mozilla.org/xre/app-info;1"]
+      .getService(Components.interfaces.nsIXULAppInfo);
+    var versionChecker = Components.classes["@mozilla.org/xpcom/version-comparator;1"]
+      .getService(Components.interfaces.nsIVersionComparator);
+    return (versionChecker.compare(appInfo.version, "10.0a2") < 0)
+  };
+
+  function getToolbarTarget(targetPos, targetHeader) {
+    //targetPos = targetPos.replace('hdrToolbox.pos.', '');
+    if (pub.cannotMoveToolbox()) {
+      targetPos = "top";
+    }
+    for (let i = 0; i < gToolboxes.length; i++) {
+      if ((targetPos == gToolboxes[i].pos) &&
+          ((gToolboxes[i].header == null) || (targetHeader == gToolboxes[i].header))
+          ) {
+        return gToolboxes[i];
+      }
+    }
+    org_mozdev_compactHeader.debug.log("orient failed " + targetPos + " " + targetHeader,
+      org_mozdev_compactHeader.debug.LOGLEVEL.warn);
+    return null;
+  }
+
+
+
   pub.fillToolboxPalette = function () {
-    org.mozdev.compactHeader.debug.log("fillToolboxPalette start");
+    org_mozdev_compactHeader.debug.log("fillToolboxPalette start");
     removeButtonDispMUA();
     var hdrToolbar = document.getElementById("header-view-toolbar");
     var hdrToolbox = document.getElementById("header-view-toolbox");
@@ -111,7 +155,8 @@ org.mozdev.compactHeader.toolbar = function() {
           hdrToolbar.currentSet = currentSet;
         }
       }
-    }
+    org_mozdev_compactHeader.debug.log("fillToolboxPalette stop");
+    };
 
     var buttonsRemove = ["hdrForwardButton", "hdrArchiveButton",
                          "hdrReplyToSenderButton"];//, "hdrReplyButton",
@@ -124,28 +169,28 @@ org.mozdev.compactHeader.toolbar = function() {
         button.setAttribute("collapsed", "true");
       }
     }
-
-    var target = "CompactHeader_hdrOtherActionsButton";
-
-    var newParent = document.getElementById(target) ||
-        document.getElementById("header-view-toolbox").palette.getElementsByAttribute("id", target)[0];
-
-    if (newParent != null) {
-      var myElement;
-      myElement= document.getElementById("otherActionsPopup");
-      if (myElement) {
-        newParent.appendChild(myElement);
-      }
-    }
-    org.mozdev.compactHeader.debug.log("fillToolboxPalette stop");
+    org_mozdev_compactHeader.debug.log("fillToolboxPalette stop");
   };
 
+  pub.showOtherActionButtonMenu = function() {
+    org_mozdev_compactHeader.debug.log("showOtherActionButtonMenu start");
+    onShowOtherActionsPopup();
+    InitMessageMark();
+    for (let menu in gOtherMenuCommands) {
+      goUpdateCommand(gOtherMenuCommands[menu]);
+    }
+    org_mozdev_compactHeader.debug.log("showOtherActionButtonMenu stop");
+  }
+
   pub.setButtonStyle = function() {
-    org.mozdev.compactHeader.debug.log("setButtonStyle start");
+    org_mozdev_compactHeader.debug.log("setButtonStyle start");
     var hdrToolbar = document.getElementById("header-view-toolbar");
     var hdrToolbox = document.getElementById("header-view-toolbox");
     var buttons1 = Array.prototype.slice.call(hdrToolbar.querySelectorAll("toolbarbutton"));
-    var buttons2 = Array.prototype.slice.call(hdrToolbox.palette.querySelectorAll("toolbarbutton"));
+    var buttons2 = [];
+    if (hdrToolbox && hdrToolbox.palette) {
+      buttons2 = Array.prototype.slice.call(hdrToolbox.palette.querySelectorAll("toolbarbutton"));
+    }
     var buttons = buttons1.concat(buttons2);
     for (var i=0; i<buttons.length; i++) {
       var button = buttons[i];
@@ -175,10 +220,14 @@ org.mozdev.compactHeader.toolbar = function() {
       }
     }
 
-    org.mozdev.compactHeader.debug.log("setButtonStyle start 1");
+    org_mozdev_compactHeader.debug.log("setButtonStyle start 1");
 
     var buttons1 = Array.prototype.slice.call(hdrToolbar.querySelectorAll("toolbaritem"));
-    var buttons2 = Array.prototype.slice.call(hdrToolbox.palette.querySelectorAll("toolbaritem"));
+    var buttons2;
+    if (hdrToolbox && hdrToolbox.palette) {
+      buttons2 = Array.prototype.slice.call(hdrToolbox.palette.querySelectorAll("toolbaritem"));
+    }
+
     var buttons = buttons1.concat(buttons2);
     for (var i=0; i<buttons.length; i++) {
       var button = buttons[i];
@@ -195,20 +244,19 @@ org.mozdev.compactHeader.toolbar = function() {
       }
     }
 
-    org.mozdev.compactHeader.debug.log("setButtonStyle stop");
+    org_mozdev_compactHeader.debug.log("setButtonStyle stop");
   };
 
-  pub.toggle = function(aHeaderViewMode) {
-    org.mozdev.compactHeader.debug.log("toolbar toggle start");
+  function moveToolbox(aHeaderViewMode, cBoxId, orient) {
+    org_mozdev_compactHeader.debug.log("toolbar toggle start headermode: " + aHeaderViewMode +
+      " cBoxId: " + cBoxId + " orien: " + orient);
     var hdrToolbox = document.getElementById("header-view-toolbox");
     var hdrToolbar = document.getElementById("header-view-toolbar");
-    var strHideLabel = document.getElementById("CompactHeader_CoheHideDetailsLabel").value;
-    var strShowLabel = document.getElementById("CompactHeader_CoheShowDetailsLabel").value;
     var firstPermanentChild = hdrToolbar.firstPermanentChild;
     var lastPermanentChild = hdrToolbar.lastPermanentChild;
     if (aHeaderViewMode) {
-      var cBox = document.getElementById("CompactHeader_collapsed2LButtonBox");
-      if (cBox.parentNode.id != hdrToolbox.parentNode.id) {
+      var cBox = document.getElementById(cBoxId);
+      if ((cBox) && (cBox.parentNode.id != hdrToolbox.parentNode.id)) {
         var cloneToolboxPalette;
         var cloneToolbarset;
         if (hdrToolbox.palette) {
@@ -225,8 +273,8 @@ org.mozdev.compactHeader.toolbar = function() {
         hdrToolbar.lastPermanentChild = lastPermanentChild;
       }
     } else {
-      var cBox = document.getElementById("expandedHeaders");
-      if (cBox.parentNode.id != hdrToolbox.parentNode.id) {
+      cBox = document.getElementById(cBoxId);
+      if ((cBox) && (cBox.parentNode.id != hdrToolbox.parentNode.id)) {
         var cloneToolboxPalette;
         var cloneToolbarset;
         if (hdrToolbox.palette) {
@@ -235,7 +283,7 @@ org.mozdev.compactHeader.toolbar = function() {
         if (hdrToolbox.toolbarset) {
           cloneToolbarset = hdrToolbox.toolbarset.cloneNode(true);
         }
-        cBox.parentNode.appendChild(hdrToolbox);
+        cBox.parentNode.insertBefore(hdrToolbox, cBox);
         hdrToolbox.palette = cloneToolboxPalette;
         hdrToolbox.toolbarset = cloneToolbarset;
         hdrToolbar = document.getElementById("header-view-toolbar");
@@ -243,47 +291,57 @@ org.mozdev.compactHeader.toolbar = function() {
         hdrToolbar.lastPermanentChild = lastPermanentChild;
       }
     }
-    org.mozdev.compactHeader.debug.log("toolbar toggle stop");
+    pub.onChangeDispMUAicon();
+    hdrToolbar.setAttribute("orient", orient);
+    org_mozdev_compactHeader.debug.log("toolbar toggle stop");
   };
 
   pub.dispMUACheck = function() {
-    org.mozdev.compactHeader.debug.log("dispMUACheck start");
+    org_mozdev_compactHeader.debug.log("dispMUACheck start");
     var dispMUAButton = document.getElementById("CompactHeader_button-dispMUA");
     var dispMUABox = document.getElementById("dispMUA");
     if (dispMUABox) {
-      dispMUABox.setAttribute("collapsed", "true"); // hide original
+      hideDispMUABox(dispMUABox);
       var IconContainerDispMUA = null;
       if (dispMUAButton) {
         /* expanded view 48 * 48 */
-        if (IconContainerDispMUA = document.getElementById("CompactHeader_dispMUAiconExp")) {
+        IconContainerDispMUA = document.getElementById("CompactHeader_dispMUAiconExp");
+        if (IconContainerDispMUA) {
           IconContainerDispMUA.setAttribute("collapsed", "true");
         }
         /* two line view 32 * 32 */
-        if (IconContainerDispMUA = document.getElementById("CompactHeader_dispMUAicon2line")) {
+        IconContainerDispMUA = document.getElementById("CompactHeader_dispMUAicon2line")
+        if (IconContainerDispMUA) {
           IconContainerDispMUA.setAttribute("collapsed", "true");
         }
         /* compact view 24 * 24 */
-        if (IconContainerDispMUA = document.getElementById("CompactHeader_dispMUAiconCompact")) {
+        IconContainerDispMUA = document.getElementById("CompactHeader_dispMUAiconCompact")
+        if (IconContainerDispMUA) {
           IconContainerDispMUA.setAttribute("collapsed", "true");
         }
       }
       else if (dispMUABox){
         /* expanded view 48 * 48 */
-        if (IconContainerDispMUA = document.getElementById("CompactHeader_dispMUAiconExp")) {
+        IconContainerDispMUA = document.getElementById("CompactHeader_dispMUAiconExp")
+        if (IconContainerDispMUA) {
           IconContainerDispMUA.removeAttribute("collapsed");
         }
         if (cohePrefBranch.getBoolPref("headersize.twolineview")) {
           /* two line view 32 * 32 */
-          if (IconContainerDispMUA = document.getElementById("CompactHeader_dispMUAicon2line"))
+          IconContainerDispMUA = document.getElementById("CompactHeader_dispMUAicon2line")
+          if (IconContainerDispMUA)
             IconContainerDispMUA.removeAttribute("collapsed");
-          if (IconContainerDispMUA = document.getElementById("CompactHeader_dispMUAiconCompact"))
+          IconContainerDispMUA = document.getElementById("CompactHeader_dispMUAiconCompact")
+          if (IconContainerDispMUA)
             IconContainerDispMUA.setAttribute("collapsed", "true");
         }
         else {
           /* compact view 24 * 24 */
-          if (IconContainerDispMUA = document.getElementById("CompactHeader_dispMUAiconCompact"))
+          IconContainerDispMUA = document.getElementById("CompactHeader_dispMUAiconCompact")
+          if (IconContainerDispMUA)
             IconContainerDispMUA.removeAttribute("collapsed");
-          if (IconContainerDispMUA = document.getElementById("CompactHeader_dispMUAicon2line"))
+          IconContainerDispMUA = document.getElementById("CompactHeader_dispMUAicon2line")
+          if (IconContainerDispMUA)
             IconContainerDispMUA.setAttribute("collapsed", "true");
         }
       }
@@ -291,64 +349,66 @@ org.mozdev.compactHeader.toolbar = function() {
     else {
       var IconContainerDispMUA = null;
       /* expanded view 48 * 48 */
-      if (IconContainerDispMUA = document.getElementById("CompactHeader_dispMUAiconExp")) {
+      IconContainerDispMUA = document.getElementById("CompactHeader_dispMUAiconExp")
+      if (IconContainerDispMUA) {
         IconContainerDispMUA.setAttribute("collapsed", "true");
       }
       /* two line view 32 * 32 */
-      if (IconContainerDispMUA = document.getElementById("CompactHeader_dispMUAicon2line")) {
+      IconContainerDispMUA = document.getElementById("CompactHeader_dispMUAicon2line")
+      if (IconContainerDispMUA) {
         IconContainerDispMUA.setAttribute("collapsed", "true");
       }
       /* compact view 24 * 24 */
-      if (IconContainerDispMUA = document.getElementById("CompactHeader_dispMUAiconCompact")) {
+      IconContainerDispMUA = document.getElementById("CompactHeader_dispMUAiconCompact")
+      if (IconContainerDispMUA) {
         IconContainerDispMUA.setAttribute("collapsed", "true");
       }
     }
-    org.mozdev.compactHeader.debug.log("dispMUACheck stop");
+    org_mozdev_compactHeader.debug.log("dispMUACheck stop");
   };
 
   pub.onChangeDispMUAicon = function() {
-    org.mozdev.compactHeader.debug.log("onChangeDispMUAicon start");
-    var imageSrc = document.getElementById("dispMUAicon").getAttribute("src");
-    var IconContainerDispMUA = null;
-    /* toolbar button */
-    if (IconContainerDispMUA = document.getElementById("CompactHeader_button-dispMUA")) {
-      IconContainerDispMUA.setAttribute("image", imageSrc);
+    org_mozdev_compactHeader.debug.log("onChangeDispMUAicon start");
+    var dispMUAbroadcast = document.getElementById("dispMUAbroadcast")
+    if (dispMUAbroadcast) {
+      var imageSrc = dispMUAbroadcast.getAttribute("src");
+      org_mozdev_compactHeader.debug.log("onChangeDispMUAicon: " + imageSrc);
+      var IconContainerDispMUA = null;
+      /* toolbar button */
+      IconContainerDispMUA = document.getElementById("CompactHeader_button-dispMUA")
+      if (IconContainerDispMUA) {
+        IconContainerDispMUA.setAttribute("image", imageSrc);
+      }
     }
-    /* expanded view 48 * 48 */
-    if (IconContainerDispMUA = document.getElementById("CompactHeader_dispMUAiconExp")) {
-      IconContainerDispMUA.setAttribute("src", imageSrc);
+    org_mozdev_compactHeader.debug.log("onChangeDispMUAicon stop");
+  };
+
+  function hideDispMUABox(dispMUABox) {
+    org_mozdev_compactHeader.debug.log("hideDispMUABox start");
+    dispMUABox.setAttribute("collapsed", "true"); // hide original
+    var dispMUAicon = document.getElementById("dispMUAicon");
+    var messengerWindow = document.getElementById("messengerWindow");
+    if (dispMUAicon && messengerWindow && dispMUAicon.localName == "image") {
+      org_mozdev_compactHeader.debug.log("hideDispMUABox 1");
+      var broadcasterset = document.createElement("broadcasterset");
+      messengerWindow.appendChild(broadcasterset);
+      org_mozdev_compactHeader.debug.log("hideDispMUABox 2");
+      dispMUAicon.parentNode.removeChild(dispMUAicon);
+      org_mozdev_compactHeader.debug.log("hideDispMUABox 3");
+      var dispMUAbroadcast = document.createElement("broadcaster");
+      org_mozdev_compactHeader.debug.log("hideDispMUABox 4");
+      dispMUAbroadcast.id = "dispMUAicon";
+      //dispMUAbroadcast.setAttribute("src", "");
+      dispMUAbroadcast.setAttribute("tooltiptext", "");
+      org_mozdev_compactHeader.debug.log("hideDispMUABox 5");
+      broadcasterset.appendChild(dispMUAbroadcast);
+      org_mozdev_compactHeader.debug.log("hideDispMUABox 6");
     }
-    /* two line view 32 * 32 */
-    if (IconContainerDispMUA = document.getElementById("CompactHeader_dispMUAicon2line")) {
-      IconContainerDispMUA.setAttribute("src", imageSrc);
-    }
-    /* compact view 24 * 24 */
-    if (IconContainerDispMUA = document.getElementById("CompactHeader_dispMUAiconCompact")) {
-      IconContainerDispMUA.setAttribute("src", imageSrc);
-    }
-    org.mozdev.compactHeader.debug.log("onChangeDispMUAicon 1");
-    var tooltipText = document.getElementById("dispMUAicon").getAttribute("tooltiptext");
-    var buttonDispMUA = document.getElementById("CompactHeader_button-dispMUA");
-    if (buttonDispMUA) {
-      buttonDispMUA.setAttribute("tooltiptext", tooltipText);
-    }
-    /* expanded view 48 * 48 */
-    if (IconContainerDispMUA = document.getElementById("CompactHeader_dispMUAiconExp")) {
-      IconContainerDispMUA.setAttribute("tooltiptext", tooltipText);
-    }
-    /* two line view 32 * 32 */
-    if (IconContainerDispMUA = document.getElementById("CompactHeader_dispMUAicon2line")) {
-      IconContainerDispMUA.setAttribute("tooltiptext", tooltipText);
-    }
-    /* compact view 24 * 24 */
-    if (IconContainerDispMUA = document.getElementById("CompactHeader_dispMUAiconCompact")) {
-      IconContainerDispMUA.setAttribute("tooltiptext", tooltipText);
-    }
-    org.mozdev.compactHeader.debug.log("onChangeDispMUAicon stop");
+    org_mozdev_compactHeader.debug.log("hideDispMUABox stop");
   };
 
   function removeButtonDispMUA() {
-    org.mozdev.compactHeader.debug.log("removeButtonDispMUA start");
+    org_mozdev_compactHeader.debug.log("removeButtonDispMUA start");
     if (!document.getElementById("dispMUA")) {
       var button = document.getElementById("CompactHeader_button-dispMUA");
       if (button) {
@@ -367,7 +427,7 @@ org.mozdev.compactHeader.toolbar = function() {
         button2.parentNode.removeChild(button2);
       }
     }
-    org.mozdev.compactHeader.debug.log("removeButtonDispMUA stop");
+    org_mozdev_compactHeader.debug.log("removeButtonDispMUA stop");
   };
 
   function addClass(el, strClass) {
@@ -448,31 +508,162 @@ org.mozdev.compactHeader.toolbar = function() {
   };
 
   pub.populateEmptyToolbar = function() {
-    org.mozdev.compactHeader.debug.log("start populateEmptyToolbar");
+    org_mozdev_compactHeader.debug.log("start populateEmptyToolbar");
     var hdrToolbar = document.getElementById("header-view-toolbar");
     if (hdrToolbar) {
-      org.mozdev.compactHeader.debug.log("populateEmptyToolbar 1");
+      org_mozdev_compactHeader.debug.log("populateEmptyToolbar 1");
       if (hdrToolbar.currentSet == "__empty") {
-        org.mozdev.compactHeader.debug.log("populateEmptyToolbar 2");
+        org_mozdev_compactHeader.debug.log("populateEmptyToolbar 2");
         pub.CHTSetDefaultButtons();
       }
     }
-    org.mozdev.compactHeader.debug.log("stop populateEmptyToolbar");
+    org_mozdev_compactHeader.debug.log("stop populateEmptyToolbar");
   };
 
   pub.onDoCustomizationHeaderViewToolbox = function(event) {
-    org.mozdev.compactHeader.debug.log("onDoCustomizationHeaderViewToolbox start" + event);
-    org.mozdev.compactHeader.toolbar.CHTUpdateReplyButton();
-    org.mozdev.compactHeader.toolbar.CHTUpdateJunkButton();
-    org.mozdev.customizeHeaderToolbar.messenger.saveToolboxData();
-    org.mozdev.compactHeader.toolbar.dispMUACheck();
-    org.mozdev.compactHeader.buttons.coheToggleStar();
+    org_mozdev_compactHeader.debug.log("onDoCustomizationHeaderViewToolbox start" + event);
+    org_mozdev_compactHeader.toolbar.CHTUpdateReplyButton();
+    org_mozdev_compactHeader.toolbar.CHTUpdateJunkButton();
+    org_mozdev_compactHeader.messenger.saveToolboxData();
+    org_mozdev_compactHeader.toolbar.dispMUACheck();
+    org_mozdev_compactHeader.buttons.coheToggleStar();
     var dispMUAicon = document.getElementById("dispMUAicon");
     if (dispMUAicon) {
-      org.mozdev.compactHeader.toolbar.onChangeDispMUAicon();
+      org_mozdev_compactHeader.toolbar.onChangeDispMUAicon();
     }
-    org.mozdev.compactHeader.debug.log("onDoCustomizationHeaderViewToolbox done");
+    org_mozdev_compactHeader.debug.log("onDoCustomizationHeaderViewToolbox done");
   };
 
+  var setToolboxRunning = false;
+  var currentToolboxPosition;
+  var currentToolboxType;
+  var currentHeaderViewMode;
+
+  pub.setCurrentToolboxPosition = function(aHeaderViewMode) {
+    var targetType = "single";
+    if (setToolboxRunning) {
+      org_mozdev_compactHeader.debug.log("setCurrentToolboxPosition is running");
+      return;
+    }
+    setToolboxRunning = true;
+
+    org_mozdev_compactHeader.debug.log("setCurrentToolboxPosition start");
+
+    var singleMessage = document.getElementById("singlemessage");
+    var targetPos = cohePrefBranch.getCharPref("toolbox.position");
+    var multiMessage = document.getElementById("multimessage");
+    var multiBBox;
+
+    if (singleMessage && singleMessage.getAttribute("hidden")) {
+      targetType = "multi";
+    }
+
+    org_mozdev_compactHeader.debug.log("setCurrentToolboxPosition 1");
+
+    if ((currentToolboxPosition == targetPos) &&
+        (currentToolboxType == targetType) &&
+        (currentHeaderViewMode == aHeaderViewMode) &&
+        (targetType == "single")) {
+      org_mozdev_compactHeader.debug.log("curPos: " + currentToolboxPosition + " targetPos: " + targetPos);
+      org_mozdev_compactHeader.debug.log("curType: " + currentToolboxType + " targetType: " + targetType);
+      org_mozdev_compactHeader.debug.log("setCurrentToolboxPosition does not need to change position/type");
+      setToolboxRunning = false;
+      return;
+    }
+
+    org_mozdev_compactHeader.debug.log("setCurrentToolboxPosition 2");
+
+    currentToolboxPosition = targetPos;
+    currentToolboxType = targetType;
+    currentHeaderViewMode = aHeaderViewMode;
+
+    org_mozdev_compactHeader.debug.log("setCurrentToolboxPosition 3");
+
+    if (multiMessage){
+      org_mozdev_compactHeader.debug.log("multiMessage " + multiMessage);
+      try {
+        multiBBox = multiMessage.contentDocument.getElementById("header-view-toolbox");
+        org_mozdev_compactHeader.debug.log("setCurrentToolboxPosition have multiMessage.contentDocument");
+      }
+      catch (e) {
+        org_mozdev_compactHeader.debug.log("setCurrentToolboxPosition no multiMessage.contentDocument " + e);
+      }
+    }
+
+    org_mozdev_compactHeader.debug.log("setCurrentToolboxPosition mid 0");
+
+    var hdrViewToolbox = document.getElementById("header-view-toolbox");
+    if (!hdrViewToolbox) {
+      org_mozdev_compactHeader.debug.log("no header-view-toolbox!",
+        org_mozdev_compactHeader.debug.LOGLEVEL.warn);
+      setToolboxRunning = false;
+      return;
+    }
+
+    org_mozdev_compactHeader.debug.log("setCurrentToolboxPosition mid 1");
+
+    if (targetPos == "none") {
+      hdrViewToolbox.setAttribute("collapsed", "true");
+      if (multiBBox) {
+        multiBBox.setAttribute("collapsed", "true");
+      }
+      org_mozdev_compactHeader.debug.log("none stop");
+      setToolboxRunning = false;
+      return;
+    }
+    else {
+      hdrViewToolbox.removeAttribute("collapsed");
+      if (multiBBox) {
+        multiBBox.removeAttribute("collapsed");
+      }
+    }
+
+    org_mozdev_compactHeader.debug.log("setCurrentToolboxPosition mid 2");
+
+    if (multiBBox) {
+      multiBBox.removeAttribute("collapsed");
+    }
+
+    if (singleMessage && singleMessage.getAttribute("hidden")) {
+      // The multi message view is visible
+      org_mozdev_compactHeader.debug.log("move to multibuttonhbox");
+      var targetToolbox = getToolbarTarget(targetPos, "");
+      org_mozdev_compactHeader.debug.log("move to multibuttonhbox 1");
+      if (multiBBox) {
+        if (targetPos != "top") {
+          org_mozdev_compactHeader.debug.log("x multiBBox: "+multiBBox);
+          multiBBox.setAttribute("collapsed", "true");
+          hdrViewToolbox.removeAttribute("collapsed");
+        } else {
+          org_mozdev_compactHeader.debug.log("x multiBBox: "+multiBBox);
+          multiBBox.removeAttribute("collapsed");
+          hdrViewToolbox.setAttribute("collapsed", "true");
+        }
+      }
+      if (targetToolbox) {
+        moveToolbox(aHeaderViewMode, targetToolbox.id, targetToolbox.orient);
+      }
+    }
+    else {
+      // The single message view is visible
+      org_mozdev_compactHeader.debug.log("move to singlemessage");
+      var targetToolbox;
+      if (aHeaderViewMode) {
+        org_mozdev_compactHeader.debug.log("setCurrentToolboxPosition aHeaderViewMode");
+        targetToolbox = getToolbarTarget(targetPos, "compact");
+      }
+      else {
+        org_mozdev_compactHeader.debug.log("setCurrentToolboxPosition no aHeaderViewMode");
+        targetToolbox = getToolbarTarget(targetPos, "expanded");
+      }
+      if (targetToolbox) {
+        moveToolbox(aHeaderViewMode, targetToolbox.id, targetToolbox.orient);
+      }
+    }
+    org_mozdev_compactHeader.debug.log("setCurrentToolboxPosition stop");
+    setToolboxRunning = false;
+  }
+
+
   return pub;
 }();
